import asyncio
from importlib import resources
import logging
from logging import config
from contextlib import asynccontextmanager
import os
import threading

from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware
from fastapi.responses import FileResponse
from fastapi.staticfiles import StaticFiles

from aiconsole.agents.agents import agents
from aiconsole.api.routers import app_router
from aiconsole.materials.materials import materials
from aiconsole.settings import settings, log_config
from aiconsole.utils.initialize_project_directory import initialize_project_directory

from aiconsole.utils.is_update_needed import is_update_needed
from aiconsole.utils.connection_manager import connection_manager, NotificationWSMessage


@asynccontextmanager
async def lifespan(app: FastAPI):
    await materials.reload()
    await agents.reload()
    yield

config.dictConfig(log_config)

log = logging.getLogger(__name__)


def app():
    initialize_project_directory()

    app = FastAPI(title="AI Console", lifespan=lifespan)

    app.add_middleware(
        CORSMiddleware,
        allow_origins=settings.ORIGINS,
        allow_credentials=True,
        allow_methods=["*"],
        allow_headers=["*"],
    )

    app.include_router(app_router)

    @app.get("/")
    def root():
        return FileResponse(str(resources.path('aiconsole.static', 'index.html')))

    @app.get("/chats/{chat_id}")
    def chats(chat_id: str):
        return FileResponse(str(resources.path('aiconsole.static', 'index.html')))

    log.info(
        f"Static files directory: {str(resources.path('aiconsole', 'static'))}")

    app.mount("/", StaticFiles(directory=str(resources.path('aiconsole', 'static'))))

    if is_update_needed():
        threading.Timer(5, lambda: asyncio.run(connection_manager.send_message_to_all(NotificationWSMessage(
            title="Update available", message="A new version of AI Console is available.")))).start()

    return app
